-- Test tax
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(35);

set search_path to numerus, auth, public;

select has_table('tax');
select has_pk('tax' );
select table_privs_are('tax', 'guest', array []::text[]);
select table_privs_are('tax', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('tax', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('tax', 'authenticator', array []::text[]);

SELECT has_sequence('tax_tax_id_seq');
SELECT sequence_privs_are('tax_tax_id_seq', 'guest', array[]::text[]);
SELECT sequence_privs_are('tax_tax_id_seq', 'invoicer', array['USAGE']);
SELECT sequence_privs_are('tax_tax_id_seq', 'admin', array['USAGE']);
SELECT sequence_privs_are('tax_tax_id_seq', 'authenticator', array[]::text[]);

select has_column('tax', 'tax_id');
select col_is_pk('tax', 'tax_id');
select col_type_is('tax', 'tax_id', 'integer');
select col_not_null('tax', 'tax_id');
select col_has_default('tax', 'tax_id');
select col_default_is('tax', 'tax_id', 'nextval(''tax_tax_id_seq''::regclass)');

select has_column('tax', 'company_id');
select col_is_fk('tax', 'company_id');
select fk_ok('tax', 'company_id', 'company', 'company_id');
select col_type_is('tax', 'company_id', 'integer');
select col_not_null('tax', 'company_id');
select col_hasnt_default('tax', 'company_id');

select has_column('tax', 'name');
select col_type_is('tax', 'name', 'text');
select col_not_null('tax', 'name');
select col_hasnt_default('tax', 'name');

select has_column('tax', 'rate');
select col_type_is('tax', 'rate', 'tax_rate');
select col_not_null('tax', 'rate');
select col_hasnt_default('tax', 'rate');


set client_min_messages to warning;
truncate tax cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD')
;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into tax (company_id, name, rate)
values (2, 'VAT 21 %', 0.21)
     , (2, 'IRPF -15 %', -0.15)
     , (4, 'VAT 21 %', 0.21)
     , (4, 'VAT 10 %', 0.10)
     , (4, 'VAT 5 %', 0.05)
     , (4, 'VAT 4 %', 0.04)
     , (4, 'VAT 0 %', 0.00)
;

prepare tax_data as
select company_id, name, rate
from tax
order by company_id, rate;

set role invoicer;
select is_empty('tax_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'tax_data',
	$$ values ( 2, 'IRPF -15 %', -0.15::tax_rate )
	        , ( 2, 'VAT 21 %', 0.21::tax_rate )
	$$,
	'Should only list taxes of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'tax_data',
	$$ values (4, 'VAT 0 %', 0.00::tax_rate)
	        , (4, 'VAT 4 %', 0.04::tax_rate)
	        , (4, 'VAT 5 %', 0.05::tax_rate)
	        , (4, 'VAT 10 %', 0.10::tax_rate)
	        , (4, 'VAT 21 %', 0.21::tax_rate)
	$$,
	'Should only list taxes of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'tax_data',
	'42501', 'permission denied for table tax',
	'Should not allow select to guest users'
);
reset role;


select *
from finish();

rollback;

